import { newPromise } from "../utils";

type flashcardChangedCallback = (
	card: Flashcard,
	change: "added" | "removed"
) => void;
export class Flashcards {
	db: Promise<IDBDatabase>;
	private callbacks: flashcardChangedCallback[] = [];

	constructor() {
		const [promise, resolve, reject] = newPromise<IDBDatabase>();
		this.db = promise;

		const req = indexedDB.open("FlashCardDB", 1);
		req.onupgradeneeded = () => {
			const objectStore = req.result.createObjectStore("flashcards", {
				keyPath: "id",
				autoIncrement: true,
			});
			objectStore.createIndex("src", "src");
			objectStore.createIndex("result", "result");
			objectStore.createIndex("dateAdded", "dateAdded");
			objectStore.createIndex("exported", "exported");
			objectStore.createIndex("srcLang", "languages.srcLang.code");
			objectStore.createIndex("dstLang", "languages.dstLang.code");
		};
		req.onsuccess = () => resolve(req.result);
		req.onerror = () =>
			reject(
				`Error initializing the database backend, ${req.error}. This shouldn't ever happen`
			);
	}

	async addFlashcard(t: Translation | Flashcard): Promise<Flashcard> {
		let card: Omit<Flashcard, "id"> & { id?: IDBValidKey };
		if ("id" in t) card = t;
		else {
			card = {
				...t,
				dateAdded: new Date(),
				exported: false,
			};
		}

		const req = (await this.db)
			.transaction(["flashcards"], "readwrite")
			.objectStore("flashcards")
			.put(card);

		const [promise, resolve, reject] = newPromise<Flashcard>();
		req.onsuccess = () => {
			const newCard = {
				id: req.result,
				...card,
			};
			resolve(newCard);
			this.callbacks.forEach((c) => c(newCard, "added"));
		};
		req.onerror = () => reject(req.error);
		return promise;
	}

	async removeFlashcard(card: Flashcard): Promise<void> {
		if (!card.id) return Promise.reject("Undefined ID");
		const req = (await this.db)
			.transaction(["flashcards"], "readwrite")
			.objectStore("flashcards")
			.delete(card.id);

		const [promise, resolve, reject] = newPromise<void>();
		req.onsuccess = () => {
			resolve();
			this.callbacks.forEach((c) => c(card, "removed"));
		};
		req.onerror = () => reject(req.error);
		return promise;
	}
	async getCard(cardID: IDBValidKey): Promise<Flashcard | undefined> {
		const req = (await this.db)
			.transaction(["flashcards"], "readonly")
			.objectStore("flashcards")
			.get(cardID);

		const [promise, resolve, reject] = newPromise<Flashcard | undefined>();
		req.onsuccess = () => resolve(req.result);
		req.onerror = () => reject(req.error);
		return promise;
	}

	async getAllCards(): Promise<Array<Flashcard>> {
		const req = (await this.db)
			.transaction(["flashcards"], "readonly")
			.objectStore("flashcards")
			.getAll();

		const [promise, resolve, reject] = newPromise<Flashcard[]>();
		req.onsuccess = () => resolve(req.result);
		req.onerror = () => reject(req.error);
		return promise;
	}

	async getAllKeys(): Promise<IDBValidKey[]> {
		const req = (await this.db)
			.transaction(["flashcards"], "readonly")
			.objectStore("flashcards")
			.getAllKeys();

		const [promise, resolve, reject] = newPromise<IDBValidKey[]>();
		req.onsuccess = () => resolve(req.result);
		req.onerror = () => reject(req.error);
		return promise;
	}

	async getCardFromTransRequest(src: string, language: LanguagePair) {
		const [promise, resolve, reject] = newPromise<Flashcard | undefined>();

		const req = (await this.db)
			.transaction(["flashcards"], "readonly")
			.objectStore("flashcards")
			.index("src")
			.openCursor(IDBKeyRange.only(src));
		req.onsuccess = () => {
			const cursor = req.result;
			const res = cursor?.value as Flashcard | undefined;

			if (
				res?.languages?.srcLang?.code == language.srcLang.code &&
				res?.languages?.dstLang?.code == language.dstLang.code
			)
				resolve(res);
			else if (cursor) cursor.continue();
			else resolve(undefined);
		};
		req.onerror = reject;
		return promise;
	}
	addChangeCallback(callback: flashcardChangedCallback) {
		this.callbacks.push(callback);
		console.log(this.callbacks);
	}

	removeChangeCallback(callback: flashcardChangedCallback) {
		this.callbacks = this.callbacks.filter((f) => f != callback);
	}
}
